package core.structures;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.StringTokenizer;

/**
 * 
 * <pre>
 *  Copyright (c) 2006 Dominik Schulz
 *  Copyright (c) 2006 Florian Lindner
 *  Copyright (c) 2006 Philip Hartmann
 *  
 *  This file is part of jPOPc.
 *
 *  jPOPc is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jPOPc is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with jPOPc; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *  
 * 
 * This class stores our webserver config. Since we will have more configuration
 * options than a simple port I think this would make sense.
 * 
 * This is a class that stores runtime configuration and common constants.
 * 
 * It also handles reading the configuration-file.
 * 
 * @author tex
 * 
 */
public class Config {

	public static final String	CRLF				= "\r\n";

	private static final String	PRODUCT				= "jPOPc";

	private static final String	VERSION				= "0.1a";

	public static final String	SIGNATURE			= PRODUCT + " " + VERSION;

	public static final String	POP3_STATUS_SUCCESS	= "+OK";

	public static final String	POP3_STATUS_ERROR	= "-ERR";

	public static final String	POP3_CMD_USER		= "USER";

	public static final String	POP3_CMD_PASS		= "PASS";

	public static final String	POP3_CMD_STAT		= "STAT";

	public static final String	POP3_CMD_LIST		= "LIST";

	public static final String	POP3_CMD_RETR		= "RETR";

	public static final String	POP3_CMD_DELE		= "DELE";

	public static final String	POP3_CMD_NOOP		= "NOOP";

	public static final String	POP3_CMD_RSET		= "RSET";

	public static final String	POP3_CMD_QUIT		= "QUIT";

	public static final long	RETRY_WAIT			= 50;

	public static final int		GUI_UPDATE_INTERVAL	= 6543;

	public static final String	CONFIG_FILE			= "jpopc.conf";

	public static final String	CACHE_BASE_FILENAME	= System.getProperty("java.io.tmpdir")
															+ System.getProperty("file.separator") + "jpopc-tmp-";

	public static final String	CACHE_EXTENSION		= ".eml";

	private int					port				= -1;

	private InetAddress			address				= null;

	private String				username			= "";

	private String				password			= "";

	// public static final String CONFIG_FILE = "local.conf";

	private int					connectRetries		= 1;

	private boolean				deleteAfterDownload	= false;

	private boolean				cacheMails			= true;

	/**
	 * @return the address
	 */
	public InetAddress getAddress() {
		return address;
	}

	/**
	 * @return the connect_retries
	 */
	public int getConnectRetries() {
		return connectRetries;
	}

	/**
	 * @return the password
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * @return the port
	 */
	public int getPort() {
		return port;
	}

	/**
	 * @return the username
	 */
	public String getUsername() {
		return username;
	}

	/**
	 * @return the cacheMails
	 */
	public boolean isCacheMails() {
		return cacheMails;
	}

	/**
	 * @return the deleteAfterDownload
	 */
	public boolean isDeleteAfterDownload() {
		return deleteAfterDownload;
	}

	/**
	 * reads the stored config-file and sets the values
	 * 
	 * @param configFile
	 * @return true if succesfull
	 * @throws IOException
	 */
	public boolean readConfig(String configFile) throws IOException {
		BufferedReader in = new BufferedReader(new FileReader(configFile));
		String currentLine = "";
		while ((currentLine = in.readLine()) != null) {
			if (!currentLine.startsWith("#")) { // skip comments
				StringTokenizer tokens = new StringTokenizer(currentLine);
				String currentToken = "";
				if (tokens.hasMoreTokens())
					currentToken = tokens.nextToken();
				if (currentToken.equals("Address")) {
					this.setAddress(tokens.nextToken());
				} else if (currentToken.equals("Port")) {
					this.setPort(Integer.parseInt(tokens.nextToken()));
				} else if (currentToken.equals("Username")) {
					this.setUsername(tokens.nextToken());
				} else if (currentToken.equals("CacheMails") && tokens.nextToken().trim().equals("No".toLowerCase())) {
					this.setCacheMails(false);
				} else if (currentToken.equals("Password")) {
					this.setPassword(tokens.nextToken());
				} else if (currentToken.equals("ConnectRetries")) {
					this.setConnectRetries(Integer.parseInt(tokens.nextToken()));
				}
			}
		}
		return true;
	}

	/**
	 * @param address
	 *            the address to set
	 */
	public void setAddress(InetAddress address) {
		this.address = address;
	}

	/**
	 * @param address
	 *            the address to set
	 * @throws UnknownHostException
	 */
	public void setAddress(String address) throws UnknownHostException {
		this.address = InetAddress.getByName(address);
	}

	/**
	 * @param cacheMails
	 *            the cacheMails to set
	 */
	public void setCacheMails(boolean cacheMails) {
		this.cacheMails = cacheMails;
	}

	/**
	 * @param connect_retries
	 *            the connect_retries to set
	 */
	public void setConnectRetries(int connectRetries) {
		this.connectRetries = connectRetries;
	}

	/**
	 * @param deleteAfterDownload
	 *            the deleteAfterDownload to set
	 */
	public void setDeleteAfterDownload(boolean deleteAfterDownload) {
		this.deleteAfterDownload = deleteAfterDownload;
	}

	/**
	 * @param password
	 *            the password to set
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	/**
	 * @param port
	 *            the port to set
	 */
	public void setPort(int port) {
		this.port = port;
	}

	/**
	 * @param username
	 *            the username to set
	 */
	public void setUsername(String username) {
		this.username = username;
	}

}
