package core.structures;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;
import java.util.SimpleTimeZone;
import java.util.StringTokenizer;
import java.util.TimeZone;

import org.apache.log4j.Logger;

/**
 * 
 * <pre>
 *  Copyright (c) 2006 Dominik Schulz
 *  Copyright (c) 2006 Florian Lindner
 *  Copyright (c) 2006 Philip Hartmann
 *  
 *  This file is part of jPOPc.
 *
 *  jPOPc is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jPOPc is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with jPOPc; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *  
 * This class contains the header of a mail. It's mostly a data-stucture without much intelligence except for
 * setRawHeader(String) which handles reading the whole header and parsing it according to RFC(2)822.
 * 
 * All headers mandatory headers defined in RFC2822, except for the recieve path, are stored in class variables.
 * 
 * Other interesing methods could be aTrim, which cares about trimming a string by removing whitespaces and
 * exclamationmarks, and unfoldHeaderLines which handles unfolding the header lines according to RFC2822.
 * 
 * @author tex
 */
public class Header {
	static Logger				logger							= Logger.getLogger(Header.class);

	private String				from							= "";

	public static final String	TAG_FROM						= "From:";

	private String				sender							= "";

	public static final String	TAG_SENDER						= "Sender:";

	private String				replyTo							= "";

	public static final String	TAG_REPLY_TO					= "Reply-to:";

	private String				to								= "";

	public static final String	TAG_TO							= "To:";

	private String				cc								= "";

	public static final String	TAG_CC							= "CC:";

	private String				bcc								= "";

	public static final String	TAG_BCC							= "BCC:";

	private static final String	CONTENT_TYPE_NAME				= "name=";

	private static final String	CONTENT_TYPE_FILENAME			= "filename=";

	private String				messageId						= "";

	public static final String	TAG_MESSAGE_ID					= "Message-id:";

	private String				inReplyTo						= "";

	public static final String	TAG_IN_REPLY_TO					= "in-reply-to:";

	private String				references						= "";

	public static final String	TAG_REFERENCES					= "References:";

	private String				subject							= "";

	public static final String	TAG_SUBJECT						= "Subject:";

	private String				comments						= "";

	public static final String	TAG_COMMENTS					= "Comments:";

	private String				keywords						= "";

	public static final String	TAG_KEYWORDS					= "Keywords:";

	private Date				date							= null;

	public static final String	TAG_DATE						= "Date:";

	private String				rawHeader						= "";

	private String				contentType						= "";

	public static final String	CONTENT_TYPE_DEFAULT			= "text/plain";

	public static final String	CONTENT_TYPE					= "Content-Type:";

	private String				contentTransferEncoding			= "";

	public static final String	CONTENT_TRANSFER_ENCODING		= "Content-Transfer-Encoding:";

	public static final String	CONTENT_TRANSER_ENCODING_BASE64	= "Base64";

	public static final String	CONTENT_TRANSER_ENCODING_QP		= "Quoted-printable";

	public static final String	CONTENT_TRANSER_ENCODING_7BIT	= "7bit";

	public static final String	CONTENT_TRANSER_ENCODING_8BIT	= "8bit";

	private String				contentTypeCharset				= "";

	public static final String	CONTENT_TYPE_CHARSET			= "charset=";

	public static final String	CONTENT_PREFIX_MULTIPART		= "multipart";

	private static final String	CONTENT_MULTIPART_BOUNDARY		= "boundary=";

	private static final String	CONTENT_PREFIX_CHARSET			= "charset=";

	private String				contentTypeFormat				= "";

	private String				contentTypeFilename				= "";

	public static final String	CONTENT_DISPOSITION				= "Content-Disposition:";

	public static final String	CONTENT_DISPOSITION_FILENAME	= "filename=";

	private String				contentDisposition				= "";

	private String				contentDispositionFilename		= "";

	private String				contentMultipartBoundary		= "";

	/**
	 * Two Header are equal if all of their values are equal
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object other) {
		if (other instanceof Header) {
			Header hd = (Header) other;
			if (!this.getCc().equals(hd.getCc().trim())) {
				return false;
			}
			if (!this.getContentDisposition().equals(hd.getContentDisposition().trim())) {
				return false;
			}
			if (!this.getContentDispositionFilename().equals(hd.getContentDispositionFilename().trim())) {
				return false;
			}
			if (!this.getContentMultipartBoundary().equals(hd.getContentMultipartBoundary().trim())) {
				return false;
			}
			if (!this.getContentTransferEncoding().equals(hd.getContentTransferEncoding().trim())) {
				return false;
			}
			if (!this.getContentType().equals(hd.getContentType())) {
				return false;
			}
			if (!this.getContentTypeCharset().equals(hd.getContentTypeCharset())) {
				return false;
			}
			if (!this.getContentTypeFilename().equals(hd.getContentTypeFilename())) {
				return false;
			}
			if (!this.getContentTypeFormat().equals(hd.getContentTypeFormat())) {
				return false;
			}
			if (this.getDate().compareTo(hd.getDate()) != 0) {
				return false;
			}
			if (!this.getFrom().equals(hd.getFrom())) {
				return false;
			}
			if (!this.getRawHeader().equals(hd.getRawHeader())) {
				return false;
			}
			if (!this.getSender().equals(hd.getSender())) {
				return false;
			}
			if (!this.getSubject().equals(hd.getSubject())) {
				return false;
			}
			if (!this.getTo().equals(hd.getTo())) {
				return false;
			}
			// if we get here, everything is equal
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @see java.lang.Object#hashCode() From the Java API Doc: "Note that it is generally necessary to override the
	 *      hashCode method whenever this method [equals] is overridden, so as to maintain the general contract for the
	 *      hashCode method, which states that equal objects must have equal hash codes."
	 */
	public int hashCode() {
		int hashCode = 0;
		hashCode += this.getCc().hashCode();
		hashCode += this.getContentDisposition().hashCode();
		hashCode += this.getContentDispositionFilename().hashCode();
		hashCode += this.getContentMultipartBoundary().hashCode();
		hashCode += this.getContentTransferEncoding().hashCode();
		hashCode += this.getContentType().hashCode();
		hashCode += this.getContentTypeCharset().hashCode();
		hashCode += this.getContentTypeFilename().hashCode();
		hashCode += this.getContentTypeFormat().hashCode();
		hashCode += this.getDate().hashCode();
		hashCode += this.getFrom().hashCode();
		hashCode += this.getRawHeader().hashCode();
		hashCode += this.getSender().hashCode();
		hashCode += this.getSubject().hashCode();
		hashCode += this.getTo().hashCode();
		return hashCode;
	}

	/**
	 * @return the contentMultipartBoundary
	 */
	public String getContentMultipartBoundary() {
		return contentMultipartBoundary;
	}

	/**
	 * @param contentMultipartBoundary
	 *            the contentMultipartBoundary to set
	 */
	public void setContentMultipartBoundary(String contentMultipartBoundary) {
		this.contentMultipartBoundary = contentMultipartBoundary.trim();
	}

	/**
	 * @return the contentTypeCharset
	 */
	public String getContentTypeCharset() {
		return contentTypeCharset;
	}

	/**
	 * @param contentTypeCharset
	 *            the contentTypeCharset to set
	 */
	public void setContentTypeCharset(String contentTypeCharset) {
		this.contentTypeCharset = contentTypeCharset.trim();
	}

	/**
	 * @return the contentTransferEncoding
	 */
	public String getContentTransferEncoding() {
		return contentTransferEncoding;
	}

	/**
	 * @param contentTransferEncoding
	 *            the contentTransferEncoding to set
	 */
	public void setContentTransferEncoding(String contentTransferEncoding) {
		this.contentTransferEncoding = contentTransferEncoding.trim();
	}

	/**
	 * @return the cc
	 */
	public String getCc() {
		return cc;
	}

	/**
	 * @param cc
	 *            the cc to set
	 */
	public void setCc(String cc) {
		this.cc = cc.trim();
	}

	/**
	 * @return the date
	 */
	public Date getDate() {
		if (date != null) {
			return date;
		} else {
			return new Date();
		}
	}

	/**
	 * @param date
	 *            the date to set
	 */
	public void setDate(Date date) {
		this.date = date;
	}

	/**
	 * @return the from
	 */
	public String getFrom() {
		if (from != null)
			return from;
		else
			return "";
	}

	/**
	 * @param from
	 *            the from to set
	 */
	public void setFrom(String from) {
		this.from = from.trim();
	}

	/**
	 * @return the rawHeader
	 */
	public String getRawHeader() {
		return rawHeader;
	}

	/**
	 * @param rawHeader
	 *            the rawHeader to set
	 */
	public void setRawHeader(String rawHeader) {
		this.rawHeader = rawHeader;
		StringTokenizer tokens = new StringTokenizer(rawHeader, "\n");
		String token = "";
		while (tokens.hasMoreTokens()) {
			token = tokens.nextToken();
			// tests what type the header is
			if (token.toLowerCase().startsWith(TAG_FROM.toLowerCase())) {
				this.setFrom(token.substring(TAG_FROM.length() + 1).trim());
				// start after From:
			} else if (token.toLowerCase().startsWith(TAG_SENDER.toLowerCase())) {
				this.setSender(token.substring(TAG_SENDER.length() + 1).trim());
				// start after Sender:
			} else if (token.toLowerCase().startsWith(TAG_TO.toLowerCase())) {
				this.setTo(token.substring(TAG_TO.length() + 1).trim());
				// start after To:
			} else if (token.toLowerCase().startsWith(TAG_SUBJECT.toLowerCase())) {
				this.setSubject(token.substring(TAG_SUBJECT.length() + 1).trim());
				// start after Subject:
			} else if (token.toLowerCase().startsWith(TAG_CC.toLowerCase())) {
				this.setCc(token.substring(TAG_CC.length() + 1).trim());
				// start after Cc:
			} else if (token.toLowerCase().startsWith(TAG_DATE.toLowerCase())) {
				Date d = (parseDate(token.substring(TAG_DATE.length() + 1).trim()));
				this.setDate(d);
				// make sure to only check the first occurence of the content type the others may be from the
				// attachments, i.e. don't overwrite the content type
			} else if (token.toLowerCase().startsWith(CONTENT_TYPE.toLowerCase())
					&& (this.getContentType().length() < 1 || this.getContentType().toLowerCase().equals("text/plain"))) {
				String contentType = token.substring(CONTENT_TYPE.length() + 1).trim();
				this.setContentType(contentType);
			} else if (token.toLowerCase().startsWith(CONTENT_TRANSFER_ENCODING.toLowerCase())
					&& this.getContentTransferEncoding().length() < 1) {
				this.setContentTransferEncoding(token.substring(CONTENT_TRANSFER_ENCODING.length() + 1).trim());
			} else if (token.toLowerCase().startsWith(TAG_REPLY_TO.toLowerCase())) {
				this.setReplyTo(token.substring(TAG_REPLY_TO.length() + 1).trim());
			} else if (token.toLowerCase().startsWith(TAG_BCC.toLowerCase())) {
				this.setBcc(token.substring(TAG_BCC.length() + 1).trim());
			} else if (token.toLowerCase().startsWith(TAG_MESSAGE_ID.toLowerCase())) {
				this.setMessageId(token.substring(TAG_MESSAGE_ID.length() + 1).trim());
			} else if (token.toLowerCase().startsWith(TAG_IN_REPLY_TO.toLowerCase())) {
				this.setInReplyTo(token.substring(TAG_IN_REPLY_TO.length() + 1).trim());
			} else if (token.toLowerCase().startsWith(TAG_REFERENCES.toLowerCase())) {
				this.setReferences(token.substring(TAG_REFERENCES.length() + 1).trim());
			} else if (token.toLowerCase().startsWith(TAG_COMMENTS.toLowerCase())) {
				this.setComments(token.substring(TAG_COMMENTS.length() + 1).trim());
			} else if (token.toLowerCase().startsWith(TAG_KEYWORDS.toLowerCase())) {
				this.setKeywords(token.substring(TAG_KEYWORDS.length() + 1).trim());
			} else if (token.toLowerCase().startsWith(CONTENT_DISPOSITION.toLowerCase())) {
				this.setContentDisposition(token.substring(CONTENT_DISPOSITION.length() + 1).trim());
			} else {
				; // if we don't wan't it ignore it
			}
		}
		// set a default content type, if the sender didn't specify one
		if (this.getContentType().length() < 1) {
			this.setContentType(Header.CONTENT_TYPE_DEFAULT);
		}
	}

	public static Date parseDate(String dateStr) {
		Date date = null;
		TimeZone tz = new SimpleTimeZone(0, "UTC");
		SimpleDateFormat sdf1 = new SimpleDateFormat("EE, dd MMMM yyyy HH:mm:ss zz", Locale.US);
		SimpleDateFormat sdf2 = new SimpleDateFormat("EEEE, dd-MMMM-yy HH:mm:ss zz", Locale.US);
		SimpleDateFormat sdf3 = new SimpleDateFormat("EE, MMMM d HH:mm:ss yyyy", Locale.US);
		sdf1.setTimeZone(tz);
		sdf2.setTimeZone(tz);
		sdf3.setTimeZone(tz);
		// try all three possible date formats
		try {
			date = sdf1.parse(dateStr);
		} catch (Exception e) {
			try {
				date = sdf2.parse(dateStr);
			} catch (Exception e1) {
				try {
					date = sdf3.parse(dateStr);
				} catch (Exception e2) {
					logger.warn("Unparsable Date found. Exception: " + e2.getStackTrace());
					date = new Date(); // set the time to NOW, to avoid trouble
				}
			}
		}
		return date;
	}

	/**
	 * @return the sender
	 */
	public String getSender() {
		return sender;
	}

	/**
	 * @param sender
	 *            the sender to set
	 */
	public void setSender(String sender) {
		this.sender = sender.trim();
	}

	/**
	 * @return the subject
	 */
	public String getSubject() {
		if (subject != null)
			return subject;
		else
			return "";
	}

	/**
	 * @param subject
	 *            the subject to set
	 */
	public void setSubject(String subject) {
		this.subject = subject.trim();
	}

	/**
	 * @return the to
	 */
	public String getTo() {
		if (to != null)
			return to;
		else
			return "";
	}

	/**
	 * @param to
	 *            the to to set
	 */
	public void setTo(String to) {
		this.to = to.trim();
	}

	/**
	 * @return the contentType
	 */
	public String getContentType() {
		return contentType;
	}

	/**
	 * @param contentType
	 *            the contentType to set
	 */
	public void setContentType(String contentType) {
		logger.trace("Content-Type before Parsing: " + contentType);
		// the contenttype string may look like this
		// Content-Type: multipart/mixed;boundary="------------020109030400050908020005"
		// or like this
		// Content-Type: text/plain; charset=ISO-8859-1
		StringTokenizer tokens = new StringTokenizer(contentType, ";");
		// if there is anything in this string, we'll take the next
		// token after content-type: as our content-type, but
		// we'll leave anything out thats after the ;
		if (tokens.hasMoreTokens())
			this.contentType = tokens.nextToken();
		// if there are any further tokens after the ;
		// we have a charset or boundary spec.
		while (tokens.hasMoreTokens()) {
			String nt = tokens.nextToken();
			// this is for the multipart boundary
			if (nt.toLowerCase().trim().startsWith(CONTENT_MULTIPART_BOUNDARY.toLowerCase())) {
				// prefix + " und das letzte " abschneiden
				this.setContentMultipartBoundary(aTrim(nt.substring(CONTENT_MULTIPART_BOUNDARY.length() + 1)));
				// and this is for the charset
			} else if (nt.toLowerCase().trim().startsWith(CONTENT_PREFIX_CHARSET.toLowerCase())) {
				this.setContentTypeCharset(aTrim(nt.substring(CONTENT_PREFIX_CHARSET.length() + 1)));
			} else if (nt.toLowerCase().trim().startsWith(CONTENT_TYPE_NAME.toLowerCase())) {
				this.setContentTypeFilename(aTrim(nt.substring(CONTENT_TYPE_NAME.length() + 1)));
			} else if (nt.toLowerCase().trim().startsWith(CONTENT_TYPE_FILENAME.toLowerCase())) {
				this.setContentTypeFilename(aTrim(nt.substring(CONTENT_TYPE_FILENAME.length() + 1)));
			} // if it's anything else we don't care (yet)
			// i.e. we only handle multipart content types with
			// boundary spec. and ordinary content types with
			// a charset spec
		}
	}

	/**
	 * remoes the first an the last " and trailing and leading whitspaces, if present
	 * 
	 * @param str
	 * @return
	 */
	private static String aTrim(String str) {
		str = str.trim();
		if (str.charAt(0) == '"') {
			str = str.substring(1);
		}
		if (str.charAt(str.length() - 1) == '"') {
			str = str.substring(0, str.length() - 1);
		}
		return str;
	}

	public static String[] unfoldHeaderLines(ArrayList<String> hd) {
		return unfoldHeaderLines(hd.toArray(new String[hd.size()]));
	}

	/**
	 * Unfolds the Line of a Mail acording to RFC2822 by looking for lines (inside the header) that start with one WSP a
	 * concatenating this line to the previous one
	 * 
	 * @param hd
	 * @return
	 */
	public static String[] unfoldHeaderLines(String[] hd) {
		if (hd.length > 0) {
			ArrayList<String> un = new ArrayList<String>();
			for (int i = 0; i < hd.length; i++) {
				if (hd[i].startsWith(" ") || hd[i].startsWith("\t")) {
					// unfold
					un.set(un.size() - 1, un.get(un.size() - 1) + " " + hd[i].trim());
				} else {
					// just copy
					un.add(hd[i]);
				}
			}
			return un.toArray(new String[un.size()]);
		} else {
			return hd;
		}
	}

	/**
	 * @return the contentDisposition
	 */
	public String getContentDisposition() {
		return contentDisposition;
	}

	/**
	 * @param contentDisposition
	 *            the contentDisposition to set
	 */
	public void setContentDisposition(String contentDisposition) {
		StringTokenizer tokens = new StringTokenizer(contentDisposition, ";");
		// if there is anything in this string, we'll take the next
		// token after content-type: as our content-type, but
		// we'll leave anything out thats after the ;
		if (tokens.hasMoreTokens())
			this.contentDisposition = tokens.nextToken();
		// if there are any further tokens after the ;
		// we have a charset or boundary spec.
		while (tokens.hasMoreTokens()) {
			String nt = tokens.nextToken();
			if (nt.toLowerCase().trim().startsWith(CONTENT_DISPOSITION_FILENAME.toLowerCase())) {
				// prefix + " und das letzte " abschneiden
				this.setContentDispositionFilename(aTrim(nt.substring(CONTENT_DISPOSITION_FILENAME.length() + 1)));
				// and this is for the charset
			} // if it's anything else we don't care (yet)
			// i.e. we only handle multipart content types with
			// boundary spec. and ordinary content types with
			// a charset spec
		}
	}

	/**
	 * @return the contentDispositionFilename
	 */
	public String getContentDispositionFilename() {
		return contentDispositionFilename;
	}

	/**
	 * @param contentDispositionFilename
	 *            the contentDispositionFilename to set
	 */
	public void setContentDispositionFilename(String contentDispositionFilename) {
		this.contentDispositionFilename = contentDispositionFilename.trim();
	}

	/**
	 * @return the contentTypeFilename
	 */
	public String getContentTypeFilename() {
		return contentTypeFilename;
	}

	/**
	 * @param contentTypeFilename
	 *            the contentTypeFilename to set
	 */
	public void setContentTypeFilename(String contentTypeFilename) {
		this.contentTypeFilename = contentTypeFilename.trim();
	}

	/**
	 * @return the contentTypeFormat
	 */
	public String getContentTypeFormat() {
		return contentTypeFormat;
	}

	/**
	 * @param contentTypeFormat
	 *            the contentTypeFormat to set
	 */
	public void setContentTypeFormat(String contentTypeFormat) {
		this.contentTypeFormat = contentTypeFormat.trim();
	}

	/**
	 * @return the bcc
	 */
	public String getBcc() {
		return bcc;
	}

	/**
	 * @param bcc
	 *            the bcc to set
	 */
	public void setBcc(String bcc) {
		this.bcc = bcc;
	}

	/**
	 * @return the comments
	 */
	public String getComments() {
		return comments;
	}

	/**
	 * @param comments
	 *            the comments to set
	 */
	public void setComments(String comments) {
		this.comments = comments;
	}

	/**
	 * @return the inReplyTo
	 */
	public String getInReplyTo() {
		return inReplyTo;
	}

	/**
	 * @param inReplyTo
	 *            the inReplyTo to set
	 */
	public void setInReplyTo(String inReplyTo) {
		this.inReplyTo = inReplyTo;
	}

	/**
	 * @return the keywords
	 */
	public String getKeywords() {
		return keywords;
	}

	/**
	 * @param keywords
	 *            the keywords to set
	 */
	public void setKeywords(String keywords) {
		this.keywords = keywords;
	}

	/**
	 * @return the messageId
	 */
	public String getMessageId() {
		return messageId;
	}

	/**
	 * @param messageId
	 *            the messageId to set
	 */
	public void setMessageId(String messageId) {
		this.messageId = messageId;
	}

	/**
	 * @return the references
	 */
	public String getReferences() {
		return references;
	}

	/**
	 * @param references
	 *            the references to set
	 */
	public void setReferences(String references) {
		this.references = references;
	}

	/**
	 * @return the replyTo
	 */
	public String getReplyTo() {
		return replyTo;
	}

	/**
	 * @param replyTo
	 *            the replyTo to set
	 */
	public void setReplyTo(String replyTo) {
		this.replyTo = replyTo;
	}

	public String toString() {
		String str = "";
		str += "From: " + this.getFrom() + "\n";
		str += "Sender: " + this.getSender() + "\n";
		str += "To: " + this.getTo() + "\n";
		str += "CC: " + this.getCc() + "\n";
		str += "BCC: " + this.getBcc() + "\n";
		str += "Message-Id: " + this.getMessageId() + "\n";
		str += "In-Reply-To: " + this.getInReplyTo() + "\n";
		str += "References: " + this.getReferences() + "\n";
		str += "Subject: " + this.getSubject() + "\n";
		str += "Comments: " + this.getComments() + "\n";
		str += "Keywords: " + this.getKeywords() + "\n";
		str += "Date: " + this.getDate() + "\n";
		str += "Content-Type: " + this.getContentType() + "\n";
		str += "Content-Transfer-Encoding: " + this.getContentTransferEncoding() + "\n";
		str += "Content-Type-Charset: " + this.getContentTypeCharset() + "\n";
		str += "Content-Type-Format: " + this.getContentTypeFormat() + "\n";
		str += "Content-Type-Filename: " + this.getContentTypeFilename() + "\n";
		str += "Content-Disposition: " + this.getContentDisposition() + "\n";
		str += "Content-Disposition-Filename: " + this.getContentDispositionFilename() + "\n";
		str += "Content-Multipart-Boundary: " + this.getContentMultipartBoundary() + "\n";
		return str;
	}

}
