package core.structures;

import java.util.ArrayList;

import org.apache.log4j.Logger;

/**
 * 
 * <pre>
 *  Copyright (c) 2006 Dominik Schulz
 *  Copyright (c) 2006 Florian Lindner
 *  Copyright (c) 2006 Philip Hartmann
 *  
 *  This file is part of jPOPc.
 *
 *  jPOPc is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jPOPc is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with jPOPc; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *  
 * This class is our internal representation of a arbitrary mailbox. It consists of several messages.
 * 
 * @author tex
 * 
 */
public class Mailbox {

	static Logger				logger		= Logger.getLogger(Mailbox.class);

	/**
	 * This holds all the messages fetched from the Mailserver
	 */
	private ArrayList<Message>	messages	= null;

	public Mailbox() {
		messages = new ArrayList<Message>();
	}

	/**
	 * Adds a message to the Mailbox. This method is synced. so only one process/thread at a time can write a message.
	 * 
	 * @param msg
	 *            the message to add to the mailbox
	 * @return success status of this operation
	 */
	public synchronized boolean addMessage(Message msg) {
		if (msg != null && msg.getHeader() != null) {
			Header hd = msg.getHeader();
			// removes the null lines
			if (hd.getFrom().length() < 1 && hd.getTo().length() < 1 && hd.getCc().length() < 1) {
				return false;
			}
			// prevent duplicates
			for (int i = 0; i < this.messages.size(); i++) {
				if (this.messages.get(i).equals(msg)) {
					return false;
				}
			}
			return messages.add(msg);
		} else
			return false;
	}

	/**
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object mbox) {
		// this objects can only be equal it they are from equal type
		if (mbox instanceof Mailbox) {
			Mailbox other = (Mailbox) mbox;
			for (int i = 0; i < messages.size(); i++) {
				if (!messages.get(i).equals(other))
					return false;
			}
			return true;
		} else {
			return false;
		}
	}

	/**
	 * 
	 * @param i
	 *            the message to return
	 * @return the message number i
	 */
	public Message getMessage(int i) {
		return messages.get(i);
	}

	/**
	 * Returns an array containing all of the messages in this list in the correct order; the runtime type of the
	 * returned array is that of the new message.
	 */
	public Message[] getMessages() {
		if (messages != null)
			return messages.toArray(new Message[messages.size()]);
		else
			return null;
	}

	/**
	 * This method provides an array suitable for the JTable as a data model.
	 * 
	 * @return
	 */
	public String[][] getTableData() {
		if (messages != null) {
			String[][] returned = new String[messages.size()][5];
			for (int i = 0; i < messages.size(); i++) {
				if (messages.get(i) != null) {
					returned[i][0] = messages.get(i).getHeader().getSubject(); // Subject
					returned[i][1] = messages.get(i).getHeader().getFrom(); // From
					returned[i][2] = messages.get(i).getHeader().getTo(); // To
					returned[i][3] = messages.get(i).getHeader().getDate().toString(); // Date
					returned[i][4] = new Integer(messages.get(i).getNumParts()).toString(); // Number of Parts
				} else {
					;
				}
			}
			return returned;
		} else {
			return null;
		}
	}

	/**
	 * 
	 * @return a data structure that contains the Headers of the table
	 */
	public String[] getTableHeaders() {
		String[] returned = new String[5];
		returned[0] = "Subject";
		returned[1] = "From";
		returned[2] = "To";
		returned[3] = "Date";
		returned[4] = "# Parts";
		return returned;
	}

	/**
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		int hashCode = 0;
		for (int i = 0; i < messages.size(); i++) {
			hashCode += messages.get(i).hashCode();
		}
		return hashCode;
	}

	/**
	 * 
	 * @return the number of messages inside this mailbox
	 */
	public int size() {
		return messages.size();
	}

}
