package core.threads;

import org.apache.log4j.Logger;

import core.Pop3Connection;
import core.structures.Config;
import core.structures.Mailbox;

/**
 * 
 * <pre>
 *  Copyright (c) 2006 Dominik Schulz
 *  Copyright (c) 2006 Florian Lindner
 *  Copyright (c) 2006 Philip Hartmann
 *  
 *  This file is part of jPOPc.
 *
 *  jPOPc is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jPOPc is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with jPOPc; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *  
 * This class implements fetching a mail from a pop3-server. Therefore a local config and Mailbox is used for
 * configuration and saving emails
 * 
 * @link http://de.wikipedia.org/wiki/POP3
 * @link http://www.faqs.org/rfcs/rfc1939.html
 * @author tex
 */
public class FetchMailThread implements Runnable {

	static Logger	logger	= Logger.getLogger(FetchMailThread.class);

	private Config	config	= null;

	private Mailbox	mailbox	= null;

	public FetchMailThread(Config conf, Mailbox mbx) {
		this.config = conf;
		this.mailbox = mbx;
	}

	/**
	 * When the getMails Button in the USerInterface is used the MailViewer connects to the Server by using the user
	 * account and checks out for eMails in the mailbox
	 */
	public void run() {
		int tries = 0; // how often did we try to connect
		Pop3Connection pcon = new Pop3Connection(this.config);
		while (true) { // the loop is left with a break command, so keep on
			// running until told not to do
			tries++;
			if (pcon.connect() && pcon.login()) { // connect and login
				// with the credentials
				// defined in the config
				int numberOfMails = pcon.getNumberOfMails();
				// download the mails
				for (int i = 0; i <= numberOfMails; i++) {
					if (config.isCacheMails()) {
						pcon.retrieveMailFile(i);
					} else {
						mailbox.addMessage(pcon.getMailString(i));
					}
				}
				// after everything is done disconnect
				pcon.disconnect();
				// parse the mails AFTER disconnecting
				if (config.isCacheMails()) { // if we don't cache the mails, they are already parsed.
					for (int i = 0; i <= numberOfMails; i++) {
						// since processing of some (large) mails can take
						// some time I think it's best to delegate this task
						// to seperate tasks which can do the work.
						Thread th = new Thread(new ParseMailThread(this.mailbox, i));
						th.setPriority(Thread.MIN_PRIORITY);
						th.start();
					}
				}
			} // else {
			// Only as much retries allowed as specified in the Config-file
			if (tries >= config.getConnectRetries()) {
				logger.trace("Connect retries exceded. Quitting ...");
				break;
			}
			// }
			try {
				Thread.sleep(Config.RETRY_WAIT);
				// wait some time until retry
			} catch (InterruptedException e) {
				; // don't care - we're sleeping
			}
		}
	}
}
